#ifndef _GEAR_
#define _GEAR_

#include <wgd/imodel.h>

class Gear : public wgd::IModel {
	public:
	Gear(const doste::OID &);
	~Gear() {}
	
	void initialise();
	
	virtual void update();
	virtual void draw();
	
	float radius(){ return model()->getID()["radius"]; };
	float bite()  { return model()->getID()["bite"]; };	
	float taper() { return model()->getID()["taper"]; }
	int teeth()   { return model()->getID()["teeth"]; }
	
	float rotation() { return get(wgd::ix::roll); }
	
	//if line intersects gear volume
	bool intersect(const wgd::Vector2D &pos, wgd::Vector2D &npos, wgd::Vector2D &tangent);
	
	//if point is inside volume
	bool inside(const wgd::Vector2D &);
	
	//get velocity of a point
	wgd::Vector2D velocityAt(const wgd::Vector3D &);
	
	wgd::Vector2D playerPos(const double &angle,  const double &dist);
	
	//return 2d version of position
	wgd::Vector2D centre2D();
	
	//angular velocity of the gear
	DB_SET_FLOAT(rvelocity, "rvelocity");
	DB_GET_FLOAT(rvelocity, "rvelocity");
	
	//restore force (for pendulums)
	DB_SET_FLOAT(force, "force");
	DB_GET_FLOAT(force, "force");
	
	//offset from pivot
	DB_SET_FLOAT(offset, "offset");
	DB_GET_FLOAT(offset, "offset");
	
	//platform move speed
	DB_SET_FLOAT(speed, "speed");
	DB_GET_FLOAT(speed, "speed");
	
	//plane the gear works in {0=0, 1=90, 2=180, 3=270}
	DB_SET_INT(plane, "plane");
	DB_GET_INT(plane, "plane");
	
	
	INSTANCE_LOCATION("gears");
	
	INSTANCE_CONSTRUCT(Gear);

	wgd::Vector3D test;
	wgd::Vector3D test2;
	
	private:

	//move a point by an angle
	inline wgd::Vector2D shift(const wgd::Vector2D &pos, float dist, float angle) const;
	//where two lines cross
	bool cross(wgd::Vector2D &pos, const wgd::Vector2D &a1, const wgd::Vector2D &a2, const wgd::Vector2D &b1, const wgd::Vector2D &b2);
	//if a point is above a line
	bool above(const wgd::Vector2D &pos, const wgd::Vector2D &a1, const wgd::Vector2D &a2);
	//gear line segment
	void seg(int  tooth, wgd::Vector2D &a, wgd::Vector2D &b);	
	
	//Pivot point - to draw a line...
	wgd::Vector3D pivot;

};	

#endif
