#ifndef _PLAYER_
#define _PLAYER_

#include <wgd/imodel.h>
#include "gear.h"

class Player : public wgd::IModel {
	public:
	Player(const doste::OID &);
	~Player() {}
	
	virtual void update();
	
	Gear* cPlatform(){ if(standing) return plat; else return NULL; };
	
	DB_SET_FLOAT(gravity, "gravity");
	DB_GET_FLOAT(gravity, "gravity");
	
	DB_SET_FLOAT(jumpHeight, "jumpHeight");
	DB_GET_FLOAT(jumpHeight, "jumpHeight");
	
	//current {0=0, 1=90, 2=180, 3=270}
	DB_SET_INT(plane, "plane");
	DB_GET_INT(plane, "plane");	
	
	INSTANCE_LOCATION("player");
	
	INSTANCE_CONSTRUCT(Player);
	
	DOSTE_BEGINEVENTS_INHERIT(IModel);
	DOSTE_EVENT(100, setJumpVelocity);
	DOSTE_EVENT(101, setJumpVelocity);
	DOSTE_ENDEVENTS;
	
	private:
	
	//get position in current plane
	wgd::Vector2D pos();
	void pos(const wgd::Vector2D &);
	
	bool usePlatform(Gear*, wgd::Vector2D &move);
	
	//actions
	void die();
	void jump();
	void stop();
	void goLeft();
	void goRight();
	void fall();
	void land();
	
	int direction; //facing -1 or 1
	bool canMove;  //for animation
	bool prejump;  //for animation
	bool standing; //in contact
	double pAngle; //angle with the platform
	double pRadius;//distance from centre of platform
	Gear* plat;    //current platform
	
	wgd::Vector2D vel; //velocity in 2D plane
		
	float jumpVelocity;
	
	void setJumpVelocity(DOSTE_HANDLER);
	
	//utility functions	
	float wrap (float v, float min, float max);
	float clamp (float v, float min, float max);
};

#endif
